Package: R.devices
Version: 2.17.4
Depends: R (>= 2.14.0)
Imports: grDevices, R.methodsS3 (>= 1.8.1), R.oo (>= 1.24.0), R.utils
        (>= 2.10.1), base64enc (>= 0.1-2)
Suggests: digest (>= 0.6.13), Cairo (>= 1.5-9), R.rsp
VignetteBuilder: R.rsp
Authors@R: c(person("Henrik", "Bengtsson", role=c("aut", "cre", "cph"),
                                          email = "henrikb@braju.com"))
Author: Henrik Bengtsson [aut, cre, cph]
Maintainer: Henrik Bengtsson <henrikb@braju.com>
Title: Unified Handling of Graphics Devices
Description: Functions for creating plots and image files in a unified way
    regardless of output format (EPS, PDF, PNG, SVG, TIFF, WMF, etc.). Default
    device options as well as scales and aspect ratios are controlled in a uniform
    way across all device types. Switching output format requires minimal changes
    in code. This package is ideal for large-scale batch processing, because it
    will never leave open graphics devices or incomplete image files behind, even on
    errors or user interrupts.
License: LGPL (>= 2.1)
URL: https://henrikbengtsson.github.io/R.devices/,
        https://github.com/HenrikBengtsson/R.devices
BugReports: https://github.com/HenrikBengtsson/R.devices/issues
LazyLoad: TRUE
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-07 19:52:16 UTC; hb
Repository: CRAN
Date/Publication: 2026-05-09 14:42:11 UTC
Built: R 4.6.0; ; 2026-05-09 17:42:55 UTC; unix
