

<div id="badges"><!-- pkgdown markup -->
<a href="https://CRAN.R-project.org/web/checks/check_results_R.devices.html"><img border="0" src="https://www.r-pkg.org/badges/version/R.devices" alt="CRAN check status"/></a> <a href="https://github.com/HenrikBengtsson/R.devices/actions?query=workflow%3AR-CMD-check"><img border="0" src="https://github.com/HenrikBengtsson/R.devices/actions/workflows/R-CMD-check.yaml/badge.svg?branch=develop" alt="R CMD check status"/></a>     <a href="https://app.codecov.io/gh/HenrikBengtsson/R.devices"><img border="0" src="https://codecov.io/gh/HenrikBengtsson/R.devices/branch/develop/graph/badge.svg" alt="Coverage Status"/></a> 
</div>

# R.devices: Unified Handling of Graphics Devices 


## Installation
R package R.devices is available on [CRAN](https://cran.r-project.org/package=R.devices) and can be installed in R as:
```r
install.packages("R.devices")
```


### Pre-release version

To install the pre-release version that is available in Git branch `develop` on GitHub, use:
```r
remotes::install_github("HenrikBengtsson/R.devices", ref="develop")
```
This will install the package from source.  

<!-- pkgdown-drop-below -->

