#########################################################################/**
# @RdocPackage R.devices
#
# \description{
#   @eval "getDescription(R.devices)"
# }
#
# \section{To get started}{
#   \itemize{
#    \item Vignette '\href{../doc/index.html}{R.devices overview}'
#    \item @see "toEPS", @see "toPDF", @see "toPNG", ... - evaluate
#          graphics code and atomically save plot to a figure file.
#   }
# }
#
# \section{How to cite this package}{
#   To cite this package, please use:
# \preformatted{
# @eval "paste(capture.output(toBibtex(citation(package='R.devices'))), collapse='\n')"
# }
# }
#
# \author{
#  @eval "packageDescription('R.devices')$Author".
# }
#
# \section{License}{
#  @eval "packageDescription('R.devices')$License".
# }
#*/#########################################################################
