# analysisPipelines

<details>

* Version: 1.0.2
* GitHub: https://github.com/Mu-Sigma/analysis-pipelines
* Source code: https://github.com/cran/analysisPipelines
* Date/Publication: 2020-06-12 08:00:02 UTC
* Number of recursive dependencies: 125

Run `revdep_details(, "analysisPipelines")` for more info

</details>

## In both

*   checking package dependencies ... NOTE
    ```
    Packages which this enhances but not available for checking:
      'SparkR', 'reticulate'
    ```

*   checking LazyData ... NOTE
    ```
      'LazyData' is specified without a 'data' directory
    ```

# aroma.core

<details>

* Version: 3.2.2
* GitHub: https://github.com/HenrikBengtsson/aroma.core
* Source code: https://github.com/cran/aroma.core
* Date/Publication: 2021-01-05 05:10:12 UTC
* Number of recursive dependencies: 48

Run `revdep_details(, "aroma.core")` for more info

</details>

## In both

*   checking package dependencies ... NOTE
    ```
    Packages suggested but not available for checking:
      'sfit', 'expectile', 'HaarSeg', 'mpcbs'
    ```

# eoffice

<details>

* Version: 0.2.1
* GitHub: NA
* Source code: https://github.com/cran/eoffice
* Date/Publication: 2020-11-18 21:40:02 UTC
* Number of recursive dependencies: 89

Run `revdep_details(, "eoffice")` for more info

</details>

## In both

*   checking LazyData ... NOTE
    ```
      'LazyData' is specified without a 'data' directory
    ```

# inpdfr

<details>

* Version: 0.1.11
* GitHub: https://github.com/frareb/inpdfr
* Source code: https://github.com/cran/inpdfr
* Date/Publication: 2020-01-16 12:00:02 UTC
* Number of recursive dependencies: 119

Run `revdep_details(, "inpdfr")` for more info

</details>

## In both

*   checking dependencies in R code ... NOTE
    ```
    Namespace in Imports field not imported from: ‘ca’
      All declared Imports should be used.
    ```

# PointedSDMs

<details>

* Version: 1.0.6
* GitHub: https://github.com/PhilipMostert/PointedSDMs
* Source code: https://github.com/cran/PointedSDMs
* Date/Publication: 2022-06-15 08:20:12 UTC
* Number of recursive dependencies: 131

Run `revdep_details(, "PointedSDMs")` for more info

</details>

## In both

*   checking package dependencies ... NOTE
    ```
    Package suggested but not available for checking: ‘INLA’
    ```

*   checking Rd cross-references ... NOTE
    ```
    Package unavailable to check Rd xrefs: ‘INLA’
    ```

